﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.ParameterConfigurationEncoding {
  /// <summary>
  /// An operator which manipulates two double values. TODO
  /// </summary>
  [Item("UniformDoubleValueManipulator", "An operator which manipulates two double values. TODO")]
  [StorableClass]
  public class UniformDoubleValueManipulator : Item, IDoubleValueManipulator {
    #region Constructors and Cloning
    public UniformDoubleValueManipulator() : base() { }

    [StorableConstructor]
    protected UniformDoubleValueManipulator(bool deserializing) : base(deserializing) { }
    protected UniformDoubleValueManipulator(UniformDoubleValueManipulator original, Cloner cloner) : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new UniformDoubleValueManipulator(this, cloner);
    }
    #endregion

    public void Apply(IRandom random, DoubleValue value, DoubleValueRange range) {
      ApplyStatic(random, value, range);
    }

    public static void ApplyStatic(IRandom random, DoubleValue value, DoubleValueRange range) {
      var vector = new RealVector(1);
      var bounds = new DoubleMatrix(1, 2);
      bounds[0, 0] = range.LowerBound.Value;
      bounds[0, 1] = range.UpperBound.Value;
      double val = value.Value;

      do {
        vector[0] = val;
        UniformOnePositionManipulator.Apply(random, vector, bounds);
        value.Value = vector[0];
        value.Value = range.ApplyStepSize(value.Value);
      } while (!range.IsInRange(value.Value));
    }
  }
}
